"""

MARSA
A Python library that lets you experiment with the mathematics of the Rivest–Shamir–Adleman (RSA) cryptosystem.

This project is under the MIT license.

Copyright © 2024 Justine Paul Vitan. All rights reserved.

License Information: https://github.com/jpvitan/marsa/blob/master/LICENSE
Developer's Website: https://jpvitan.com/

"""

from marsa import compute


class Key:
    def __init__(self, product: int, exponent: int):
        self.product = product
        self.exponent = exponent


class Encryptor:
    def __init__(self, public_key: Key):
        self.public_key = public_key

    def encrypt(self, message: int) -> int:
        return pow(message, self.public_key.exponent, self.public_key.product)


class Decryptor:
    def __init__(self, private_key: Key):
        self.private_key = private_key

    def decrypt(self, message: int) -> int:
        return pow(message, self.private_key.exponent, self.private_key.product)


class KeyPair:
    def __init__(self, size: int = 2048, public_exponent: int = 65537):
        first_prime = compute.generate_prime(size // 2)
        second_prime = compute.generate_prime(size // 2)

        product = first_prime * second_prime
        lambda_n = compute.lcd(first_prime - 1, second_prime - 1)

        private_exponent = (
            compute.gcd_linear_combination(public_exponent, lambda_n)[0] % lambda_n
        )

        self.public_key = Key(product, public_exponent)
        self.private_key = Key(product, private_exponent)



"""

MARSA
A Python library that lets you experiment with the mathematics of the Rivest–Shamir–Adleman (RSA) cryptosystem.

This project is under the MIT license.

Copyright © 2024 Justine Paul Vitan. All rights reserved.

License Information: https://github.com/jpvitan/marsa/blob/master/LICENSE
Developer's Website: https://jpvitan.com/

"""

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="marsa",
    version="1.0.0",
    author="Justine Paul Vitan",
    author_email="justinepaulvitan5@gmail.com",
    description=(
        "A Python library that lets you experiment with the "
        "mathematics of the Rivest–Shamir–Adleman (RSA) cryptosystem."
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jpvitan/marsa",
    packages=["marsa"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



"""

MARSA
A Python library that lets you experiment with the mathematics of the Rivest–Shamir–Adleman (RSA) cryptosystem.

This project is under the MIT license.

Copyright © 2024 Justine Paul Vitan. All rights reserved.

License Information: https://github.com/jpvitan/marsa/blob/master/LICENSE
Developer's Website: https://jpvitan.com/

"""

import unittest

from marsa import compute


class TestCompute(unittest.TestCase):
    def setUp(self):
        self.in_gcd = [(6, 8), (6, 9), (11, 17), (45, 83), (120, 130), (365, 910)]
        self.ex_out_gcd = [2, 3, 1, 1, 10, 5]

        self.in_gcd_linear_combination = self.in_gcd
        self.ex_out_gcd_linear_combination = [
            (-1, 1),
            (-1, 1),
            (-3, 2),
            (24, -13),
            (-1, 1),
            (5, -2),
        ]

        self.in_lcd = [(6, 8), (6, 9), (11, 17), (45, 83), (120, 130), (365, 910)]
        self.ex_out_lcd = [24, 18, 187, 3735, 1560, 66430]

    def tearDown(self):
        pass

    def test_gcd(self):
        for i in range(0, len(self.in_gcd)):
            self.assertEqual(
                compute.gcd(self.in_gcd[i][0], self.in_gcd[i][1]), self.ex_out_gcd[i]
            )

    def test_gcd_linear_combination(self):
        for i in range(0, len(self.in_gcd_linear_combination)):
            self.assertEqual(
                compute.gcd_linear_combination(
                    self.in_gcd_linear_combination[i][0],
                    self.in_gcd_linear_combination[i][1],
                ),
                self.ex_out_gcd_linear_combination[i],
            )

    def test_lcd(self):
        for i in range(0, len(self.in_lcd)):
            self.assertEqual(
                compute.lcd(self.in_lcd[i][0], self.in_lcd[i][1]), self.ex_out_lcd[i]
            )


if __name__ == "__main__":
    unittest.main()
